// Imperative and Object-Oriented Programming Methodology
// Uppsala University - 2018 Autumn
// Copyright © 2018 Henrik Schulze & Tobias Wrigstad
#define _GNU_SOURCE
#include <stdlib.h>
#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include "yourcode.h"

struct priority_queue
{
  link_t *first;
};

static link_t *link_create(link_t *next, void *element, int priority)
{
  link_t *link = calloc(1,sizeof(link_t));
  link->next = next;
  link->element = element;
  link->priority = priority;
  return link;
}

priority_queue_t *priority_queue_create()
{
  priority_queue_t *priority_queue = calloc(1,sizeof(priority_queue_t));
  priority_queue->first = link_create(NULL, NULL, -1); /// Dummy!!
  return priority_queue;
}

/// Insert element into priority queue according to the sort order.
void priority_queue_insert_with_priority\
  (priority_queue_t *priority_queue, void *element, int priority)
{
  /// TODO!
  link_t *cursor = priority_queue->first;
  if (cursor->next == NULL)
    { // The list is empty. Simply create and insert the new link.
      cursor->next = link_create(cursor->next, element, priority);
      return;
    }
  while (cursor->next != NULL && cursor->next->priority < priority)
    {
      cursor = cursor->next;
    }
  cursor->next = link_create(cursor->next, element, priority);
}

/// Return the number of elements in the priority queue.
/// Returnera antalet element i priority queue. Detta skall räknas ut
/// genom iteration över samtliga länkar.
int priority_queue_size(priority_queue_t *priority_queue)
{
  /// TODO!
  int result = 0;
  if (priority_queue == NULL) { return result; }
  if (priority_queue->first != NULL)
    { // The queue is NOT empty.
      for (link_t *cursor = priority_queue->first->next; \
          cursor != NULL; cursor = cursor->next)
        {
          ++result;
        }
    }
  return result;
}

/// Remove the priority queue, all links and their elements.
void priority_queue_destroy(priority_queue_t *priority_queue)
{
  /// TODO!
  if (priority_queue == NULL) { return; } // No memory to free.
  if (priority_queue->first != NULL)
    { // The queue is NOT empty.
      for (link_t *cursor = priority_queue->first->next; cursor != NULL;)
        {
          free(cursor->element);
          link_t *next_cursor = cursor->next;
          free(cursor);
          cursor = next_cursor;
        }
      free(priority_queue->first);
    }
  free(priority_queue);
  free(NULL); // Allowed but meaningless - does nothing!
}

static link_t *link_insert(link_t *dnext, link_t *sprev)
{
  if (sprev == NULL) { return NULL; }
  link_t *result = sprev->next;
  if (sprev->next == NULL) { return result; }
  sprev->next = sprev->next->next; // Remove link from the source queue.
  result->next = dnext; // Prepare the result link before returning it.
  return result;
}

/// Merge the two priority queues into dest, leaving source empty.
/// (Note: implement so that links move, not just elements!)
void priority_queue_merge(priority_queue_t *source, priority_queue_t *dest)
{
  /// TODO!
  if (source->first != NULL)
    { // The source queue is NOT empty - links need to be moved.
      link_t *sprev_crsr = source->first;
      while (sprev_crsr->next != NULL)
      // ^ Do NOT iterate here - link_insert() will iterate for you! ^
        {// For each sprev_crsr, remove next from source, insert into dest:
          link_t *dcursor = dest->first;
          if (dcursor->next == NULL)
            { // The list is empty. Simply insert the source link.
              dcursor->next = link_insert(dcursor->next, sprev_crsr);
              return;
            }
          while (dcursor->next != NULL && dcursor->next->priority < sprev_crsr->next->priority)
            {
              dcursor = dcursor->next;
            }
          dcursor->next = link_insert(dcursor->next, sprev_crsr);
        }
    }
}

/// Remove the element from the queue that has the highest priority, and return it.
link_t *priority_queue_pull_highest_priority_element(priority_queue_t *priority_queue){
  /// TODO!
  link_t *result = NULL;
  if (priority_queue == NULL) { return result; }
  if (priority_queue->first != NULL)
    { // The queue is NOT empty.
      for (link_t *cursor = priority_queue->first->next; \
          cursor != NULL; cursor = cursor->next)
        {
          result = cursor;
        }
    }
  return result;
}

/// Returns a string representation of the priority queue to be printed
/// No need to read or change this function!!
char *priority_queue_print(priority_queue_t *priority_queue, elem_to_string to_string)
{
  char *result = calloc(2048, sizeof(char));
  strcat(result, "[");
  for (link_t *cursor = priority_queue->first->next; cursor != NULL; cursor = cursor->next)
    {
      char *tmp = NULL;
      asprintf(&tmp, "(%d,%s), ", cursor->priority, to_string ? \
          to_string(cursor->element) : (char *)cursor->element);
      strcat(result, tmp);
      free(tmp);
    }
  int size = strlen(result);
  strcpy(result + (size > 1 ? size - 2 : size), "]");
  return result;
}

/// Return the first link in the priority_queue for testing
/// No need to read or change this function!!
link_t *priority_queue_first(priority_queue_t *priority_queue)
{
  return priority_queue->first->next;
}
